# example4.c
#
# This example accesses the data within each row without using
# dbbind(), and illustrates the use of row buffering.
#
# It runs a query, saves all of the returned rows (up to a maximum of
# 1000) using DB-Library row buffering, and allows the user to examine
# data rows at random.
import sys
from mssqldb import *

import exutils

TYPELEN = 2

print "Demo of SQL queries in a command batch\n"

# Initialize DB-Library. */
if not dbinit():
    sys.exit(1)

# Install the user-supplied error-handling and message-handling
# routines. They are defined at the bottom of this source file.
dberrhandle(exutils.err_handler)
dbmsghandle(exutils.msg_handler)

# Get a LOGINREC structure and fill it with the necessary login
# information.
login = dblogin()
login.dbsetluser(exutils.USER)
login.dbsetlpwd(exutils.PASSWORD)
login.dbsetlapp("example4")

dbproc = login.dbopen(exutils.SERVER)

dbproc.dbcmd("select name, type, id, crdate from sysobjects")
dbproc.dbcmd(" where type = 'S'")

# Buffer the rows so we can read them any number of times.  Passing
# '0' as the number of rows to buffer will get us default row
# buffering (currently 1000 rows).  If more than 1000 rows are
# received, this program will only save the last 1000.
#
# Note that this parameter must be passed as an ASCII string.

dbproc.dbsetopt(DBBUFFER, "0", NULL)

dbproc.dbsqlexec()

name = DataBuf(SQLCHAR, SQLMAXNAME)
type = DataBuf(SQLCHAR, TYPELEN)
id = DataBuf(SQLINT4)
dbdatebuf = DataBuf(SQLDATETIME)
datebuf = DataBuf(SQLCHAR, 64)

if dbproc.dbresults() == SUCCEED:
    # Read all of the rows into DB-Library's buffer
    while 1:
        row_code = dbproc.dbnextrow()
        if row_code == NO_MORE_ROWS:
            break
        # If DB-Library's row buffer is full, throw away the oldest
        # row, to allow the newest row to be read in.
        if row_code == BUF_FULL:
            dbproc.dbclrbuf(1)

    # Let the user view any row in the table.
    print "Type the number of the row you want to see."
    print "The first row is number 1."
    print "Asking for row 0 will terminate the program."

    while 1:
        numstr = raw_input("Row number: ")
        rownum = int(numstring)
        if rownum == 0:
            break
        # Print out the requested row.
        if dbproc.dbgetrow(rownum) == NO_MORE_ROWS:
            print "That row is not in the table."
        else:
            # Copy variable-length character data (colname).
            dbproc.dbdata(name, 1, dbproc.dbdatlen(1))
            # Copy fixed-length character data.
            dbproc.dbdata(type, 2, dbproc.dbdatlen(2))
            # Copy integer data.
            dbproc.dbdata(id, 3)

            # Print out the column headers.
            print " %-20s %10s %30s %15s" \
                  % ("       NAME         ",
                     "TYPE",
                     "             DATE             ",
                     "ID")
            print " %20s %10s %30s %15s" \
                  % ("--------------------",
                     "----",
                     "---------------------------",
                     "----")
            
            # Convert datetime data to a printable string.
            dbproc.dbdata(dbdatebuf, 4, dbproc.dbdatlen(4))
            dbproc.dbconvert(dbdatebuf, datebuf)
            print "%20s %10s %30s %15d" % (name, type, datebuf, id.get())
dbexit()
